package com.sun.jndi.ldap;

import javax.naming.event.NamingExceptionEvent;
import javax.naming.ldap.LdapName;
import javax.naming.event.NamingEvent;
import javax.naming.Binding;
import javax.naming.InterruptedNamingException;
import javax.naming.ldap.HasControls;
import javax.naming.CompositeName;
import com.sun.jndi.toolkit.ctx.Continuation;
import javax.naming.ldap.Control;
import java.io.IOException;
import javax.naming.NamingException;
import javax.naming.directory.SearchResult;
import javax.naming.NamingEnumeration;
import javax.naming.event.EventContext;
import javax.naming.event.NamingListener;
import java.util.Vector;

final class NamingEventNotifier implements Runnable
{
    private static final boolean debug = false;
    private Vector<NamingListener> namingListeners;
    private Thread worker;
    private LdapCtx context;
    private EventContext eventSrc;
    private EventSupport support;
    private NamingEnumeration<SearchResult> results;
    NotifierArgs info;
    
    NamingEventNotifier(EventSupport support, LdapCtx eventSrc, NotifierArgs info, NamingListener obj) throws NamingException {
        this.info = info;
        this.support = support;
        PersistentSearchControl persistentSearchControl;
        try {
            persistentSearchControl = new PersistentSearchControl(info.mask, true, true, true);
        }
        catch (IOException rootCause) {
            NamingException ex = new NamingException("Problem creating persistent search control");
            ex.setRootCause(rootCause);
            throw ex;
        }
        this.context = (LdapCtx)eventSrc.newInstance(new Control[] { persistentSearchControl });
        this.eventSrc = eventSrc;
        (this.namingListeners = new Vector()).addElement(obj);
        (this.worker = Obj.helper.createThread(this)).setDaemon(true);
        this.worker.start();
    }
    
    void addNamingListener(NamingListener obj) {
        this.namingListeners.addElement(obj);
    }
    
    void removeNamingListener(NamingListener obj) {
        this.namingListeners.removeElement(obj);
    }
    
    boolean hasNamingListeners() {
        return this.namingListeners.size() > 0;
    }
    
    @Override
    public void run() {
        try {
            Continuation continuation = new Continuation();
            continuation.setError(this, this.info.name);
            this.results = this.context.searchAux((this.info.name == null || this.info.name.equals("")) ? new CompositeName() : new CompositeName().add(this.info.name), this.info.filter, this.info.controls, true, false, continuation);
            ((LdapSearchEnumeration)this.results).setStartName(this.context.currentParsedDN);
            while (this.results.hasMore()) {
                SearchResult searchResult = (SearchResult)this.results.next();
                Control[] array = (searchResult instanceof HasControls) ? ((HasControls)searchResult).getControls() : null;
                if (array == null) {
                    continue;
                }
                int n = 0;
                if (n < array.length && array[n] instanceof EntryChangeResponseControl) {
                    EntryChangeResponseControl entryChangeResponseControl = (EntryChangeResponseControl)array[n];
                    long changeNumber = entryChangeResponseControl.getChangeNumber();
                    switch (entryChangeResponseControl.getChangeType()) {
                        case 1: {
                            this.fireObjectAdded(searchResult, changeNumber);
                            continue;
                        }
                        case 2: {
                            this.fireObjectRemoved(searchResult, changeNumber);
                            continue;
                        }
                        case 4: {
                            this.fireObjectChanged(searchResult, changeNumber);
                            continue;
                        }
                        case 8: {
                            this.fireObjectRenamed(searchResult, entryChangeResponseControl.getPreviousDN(), changeNumber);
                            continue;
                        }
                    }
                }
                else {
                    continue;
                }
            }
        }
        catch (InterruptedNamingException ex) {}
        catch (NamingException ex2) {
            this.fireNamingException(ex2);
            this.support.removeDeadNotifier(this.info);
        }
        finally {
            this.cleanup();
        }
    }
    
    private void cleanup() {
        try {
            if (this.results != null) {
                this.results.close();
                this.results = null;
            }
            if (this.context != null) {
                this.context.close();
                this.context = null;
            }
        }
        catch (NamingException ex) {}
    }
    
    void stop() {
        if (this.worker != null) {
            this.worker.interrupt();
            this.worker = null;
        }
    }
    
    private void fireObjectAdded(Binding newBd, long value) {
        if (this.namingListeners == null || this.namingListeners.size() == 0) {
            return;
        }
        this.support.queueEvent(new NamingEvent(this.eventSrc, 0, newBd, null, new Long(value)), this.namingListeners);
        return;
    }
    
    private void fireObjectRemoved(Binding oldBd, long value) {
        if (this.namingListeners == null || this.namingListeners.size() == 0) {
            return;
        }
        this.support.queueEvent(new NamingEvent(this.eventSrc, 1, null, oldBd, new Long(value)), this.namingListeners);
        return;
    }
    
    private void fireObjectChanged(Binding newBd, long value) {
        if (this.namingListeners == null || this.namingListeners.size() == 0) {
            return;
        }
        this.support.queueEvent(new NamingEvent(this.eventSrc, 3, newBd, new Binding(newBd.getName(), null, newBd.isRelative()), new Long(value)), this.namingListeners);
        return;
    }
    
    private void fireObjectRenamed(Binding newBd, String s, long value) {
        if (this.namingListeners == null || this.namingListeners.size() == 0) {
            return;
        }
        Object oldBd = null;
        try {
            LdapName ldapName = new LdapName(s);
            if (ldapName.startsWith(this.context.currentParsedDN)) {
                oldBd = new Binding(ldapName.getSuffix(this.context.currentParsedDN.size()).toString(), null);
            }
        }
        catch (NamingException ex) {}
        if (oldBd == null) {
            oldBd = new Binding(s, null, false);
        }
        this.support.queueEvent(new NamingEvent(this.eventSrc, 2, newBd, oldBd, new Long(value)), this.namingListeners);
        return;
    }
    
    private void fireNamingException(NamingException exc) {
        if (this.namingListeners == null || this.namingListeners.size() == 0) {
            return;
        }
        this.support.queueEvent(new NamingExceptionEvent(this.eventSrc, exc), this.namingListeners);
        return;
    }
}
